@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2019 Vaclav Slavik
+ *  Copyright (C) 2000-2020 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -34,9 +34,10 @@ class WXDLLIMPEXP_FWD_CORE wxWindow;
 /// Result of PerformUpdateFromSources()
 enum class UpdateResultReason
 {
-    Unspecified,
     CancelledByUser,
-    NoSourcesFound
+    Unspecified,
+    NoSourcesFound,
+    PermissionDenied
 };
 
 enum UpdateFlags
