@@ -910,12 +910,18 @@ bool POCatalog::Load(const wxString& po_file, int flags)
     // now that the catalog is loaded, update its items with the bookmarks
     for (unsigned i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
     {
-        if (m_header.Bookmarks[i] != -1 &&
-            m_header.Bookmarks[i] < (int)m_items.size())
+        if (m_header.Bookmarks[i] == -1)
+            continue;
+
+        if (m_header.Bookmarks[i] < (int)m_items.size())
         {
             m_items[m_header.Bookmarks[i]]->SetBookmark(
                     static_cast<Bookmark>(i));
         }
+        else // invalid bookmark
+        {
+            m_header.Bookmarks[i] = -1;
+        }
     }
 
     m_fileCRLF = GetFileCRLFFormat(f);
