@@ -30,33 +30,45 @@
 
 #include "catalog.h"
 #include "cloud_sync.h"
+#include "colorscheme.h"
 #include "concurrency.h"
 #include "customcontrols.h"
 #include "errors.h"
 #include "hidpi.h"
+#include "http_client.h"
 #include "languagectrl.h"
 #include "str_helpers.h"
 #include "utility.h"
+#include "catalog_xliff.h"
 
 #include <wx/app.h>
 #include <wx/artprov.h>
 #include <wx/button.h>
 #include <wx/choice.h>
 #include <wx/config.h>
+#include <wx/dataview.h>
 #include <wx/dialog.h>
 #include <wx/msgdlg.h>
+#include <wx/renderer.h>
 #include <wx/sizer.h>
 #include <wx/statbmp.h>
 #include <wx/stdpaths.h>
 #include <wx/weakref.h>
 #include <wx/windowptr.h>
 
+#ifdef __WXMSW__
+    #include <wx/generic/private/markuptext.h>
+#endif
+
 #if !wxCHECK_VERSION(3,1,0)
     #define CenterVertical() Center()
 #endif
 
+#include <regex>
+
 #include <boost/algorithm/string.hpp>
 
+
 CrowdinLoginPanel::CrowdinLoginPanel(wxWindow *parent, int flags)
     : wxPanel(parent, wxID_ANY),
       m_state(State::Uninitialized)
@@ -66,9 +78,9 @@ CrowdinLoginPanel::CrowdinLoginPanel(wxWindow *parent, int flags)
     SetSizer(sizer);
 
     sizer->AddSpacer(PX(10));
-    auto logo = new wxStaticBitmap(this, wxID_ANY, wxArtProvider::GetBitmap("CrowdinLogoTemplate"));
+    auto logo = new StaticBitmap(this, "CrowdinLogoTemplate");
     logo->SetCursor(wxCURSOR_HAND);
-    logo->Bind(wxEVT_LEFT_UP, [](wxMouseEvent&){ wxLaunchDefaultBrowser(CrowdinClient::WrapLink("/")); });
+    logo->Bind(wxEVT_LEFT_UP, [](wxMouseEvent&){ wxLaunchDefaultBrowser(CrowdinClient::AttributeLink("/")); });
     sizer->Add(logo, wxSizerFlags().PXDoubleBorder(wxBOTTOM));
     auto explain = new ExplanationLabel(this, _("Crowdin is an online localization management platform and collaborative translation tool. Poedit can seamlessly sync PO files managed at Crowdin."));
     sizer->Add(explain, wxSizerFlags().Expand());
@@ -164,25 +176,28 @@ void CrowdinLoginPanel::CreateLoginInfoControls(State state)
 
         case State::SignedIn:
         {
-            auto account = new wxStaticText(this, wxID_ANY, _("Signed in as:"));
-            account->SetForegroundColour(SecondaryLabel::GetTextColor());
+            auto profile = new AvatarIcon(this, wxSize(PX(48), PX(48)));
             auto name = new wxStaticText(this, wxID_ANY, m_userName);
-#ifdef __WXGTK3__
-            // This is needed to avoid missizing text with bold font. See
-            // https://github.com/vslavik/poedit/pull/411 and https://trac.wxwidgets.org/ticket/16088
-            name->SetLabelMarkup("<b>" + EscapeMarkup(m_userName) + "</b>");
-#else
-            name->SetFont(name->GetFont().Bold());
-#endif
-
             auto username = new SecondaryLabel(this, m_userLogin);
 
-            sizer->Add(account, wxSizerFlags().BORDER_MACOS(wxTOP, PX(3)));
-            sizer->AddSpacer(PX(2));
+            sizer->Add(profile, wxSizerFlags().Center());
+            sizer->AddSpacer(PX(6));
             auto box = new wxBoxSizer(wxVERTICAL);
             box->Add(name, wxSizerFlags().Left());
             box->Add(username, wxSizerFlags().Left());
-            sizer->Add(box);
+            sizer->Add(box, wxSizerFlags().Center());
+            sizer->AddSpacer(PX(6));
+
+            profile->SetUserName(m_userName);
+            if (!m_userAvatar.empty())
+            {
+                http_client::download_from_anywhere(m_userAvatar)
+                .then_on_window(profile, [=](downloaded_file f)
+                {
+                    profile->LoadIcon(f.filename());
+                });
+            }
+
             break;
         }
     }
@@ -198,6 +213,7 @@ void CrowdinLoginPanel::UpdateUserInfo()
         .then_on_window(this, [=](CrowdinClient::UserInfo u) {
             m_userName = u.name;
             m_userLogin = u.login;
+            m_userAvatar = u.avatar;
             ChangeState(State::SignedIn);
         })
         .catch_all([](dispatch::exception_ptr){});
@@ -225,7 +241,7 @@ void CrowdinLoginPanel::OnSignOut(wxCommandEvent&)
 
 LearnAboutCrowdinLink::LearnAboutCrowdinLink(wxWindow *parent, const wxString& text)
     : LearnMoreLink(parent,
-                    CrowdinClient::WrapLink("/"),
+                    CrowdinClient::AttributeLink("/"),
                     text.empty() ? (MSW_OR_OTHER(_("Learn more about Crowdin"), _("Learn More About Crowdin"))) : text)
 {
 }
@@ -235,6 +251,12 @@ LearnAboutCrowdinLink::LearnAboutCrowdinLink(wxWindow *parent, const wxString& t
 namespace
 {
 
+inline wxString GetCrowdinCacheDir()
+{
+    return CloudSyncDestination::GetCacheDir() + wxFILE_SEP_PATH + "Crowdin" + wxFILE_SEP_PATH;
+}
+
+
 class CrowdinLoginDialog : public wxDialog
 {
 public:
@@ -269,42 +291,154 @@ private:
 };
 
 
+class CrowdinFileList : public wxDataViewListCtrl
+{
+public:
+    using FileInfo = CrowdinClient::FileInfo;
+
+    CrowdinFileList(wxWindow *parent)
+        : wxDataViewListCtrl(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize,
+                             wxDV_NO_HEADER | MSW_OR_OTHER(wxBORDER_SIMPLE, wxBORDER_SUNKEN))
+    {
+    #if wxCHECK_VERSION(3,1,1)
+        SetRowHeight(PX(36));
+    #endif
+        SetMinSize(wxSize(PX(500), PX(200)));
+
+        auto renderer = new MultilineTextRenderer();
+        auto column = new wxDataViewColumn(_("File"), renderer, 0, -1, wxALIGN_NOT, wxDATAVIEW_COL_RESIZABLE);
+        AppendColumn(column, "string");
+
+        ColorScheme::SetupWindowColors(this, [=]{ RefreshFileList(); });
+    }
+
+    void ClearFiles()
+    {
+        m_files.clear();
+        DeleteAllItems();
+    }
+
+    void SetFiles(std::vector<FileInfo>& files)
+    {
+        m_files = files;
+        RefreshFileList();
+    }
+
+private:
+    void RefreshFileList()
+    {
+    #ifdef __WXGTK__
+        auto secondaryFormatting = "alpha='50%'";
+    #else
+        auto secondaryFormatting = wxString::Format("foreground='%s'", ColorScheme::Get(Color::SecondaryLabel).GetAsString(wxC2S_HTML_SYNTAX));
+    #endif
+
+        DeleteAllItems();
+
+        for (auto& f : m_files)
+        {
+        #if wxCHECK_VERSION(3,1,1)
+            wxString details;
+            if (!f.branchName.empty())
+                details += str::to_wx(f.branchName) + L" → ";
+            details += str::to_wx(f.fullPath);
+
+            wxString text = wxString::Format
+            (
+                "%s\n<small><span %s>%s</span></small>",
+                EscapeCString(f.title),
+                secondaryFormatting,
+                EscapeCString(details)
+            );
+        #else
+            wxString text = str::to_wx(f.title);
+        #endif
+
+            wxVector<wxVariant> data;
+            data.push_back({text});
+            AppendItem(data);
+        }
+    }
+
+    class MultilineTextRenderer : public wxDataViewTextRenderer
+    {
+    public:
+        MultilineTextRenderer() : wxDataViewTextRenderer()
+        {
+    #if wxCHECK_VERSION(3,1,1)
+            EnableMarkup();
+    #endif
+        }
+
+    #ifdef __WXMSW__
+        bool Render(wxRect rect, wxDC *dc, int state)
+        {
+            int flags = 0;
+            if ( state & wxDATAVIEW_CELL_SELECTED )
+                flags |= wxCONTROL_SELECTED;
+            
+            for (auto& line: wxSplit(m_text, '\n'))
+            {
+                wxItemMarkupText markup(line);
+                markup.Render(GetView(), *dc, rect, flags, GetEllipsizeMode());
+                rect.y += rect.height / 2;
+            }
+            
+            return true;
+        }
+
+        wxSize GetSize() const
+        {
+            if (m_text.empty())
+                return wxSize(wxDVC_DEFAULT_RENDERER_SIZE,wxDVC_DEFAULT_RENDERER_SIZE);
+
+            auto size = wxDataViewTextRenderer::GetSize();
+            size.y *= 2; // approximation enough for our needs
+            return size;
+        }
+    #endif // __WXMSW__
+    };
+
+private:
+    std::vector<FileInfo> m_files;
+};
+
+
 class CrowdinOpenDialog : public wxDialog
 {
 public:
-    CrowdinOpenDialog(wxWindow *parent) : wxDialog(parent, wxID_ANY, _("Open Crowdin translation"))
+    CrowdinOpenDialog(wxWindow *parent) : wxDialog(parent, wxID_ANY, _("Open Crowdin translation"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
     {
         auto topsizer = new wxBoxSizer(wxVERTICAL);
         topsizer->SetMinSize(PX(400), -1);
 
-        topsizer->AddSpacer(PX(10));
+        auto loginSizer = new wxBoxSizer(wxHORIZONTAL);
+        topsizer->AddSpacer(PX(6));
+        topsizer->Add(loginSizer, wxSizerFlags().Right().PXDoubleBorder(wxLEFT|wxRIGHT));
+        auto loginText = new SecondaryLabel(this, "");
+        auto loginImage = new AvatarIcon(this, wxSize(PX(24), PX(24)));
+        loginSizer->Add(loginText, wxSizerFlags().ReserveSpaceEvenIfHidden().Center().Border(wxRIGHT, PX(5)));
+        loginSizer->Add(loginImage, wxSizerFlags().ReserveSpaceEvenIfHidden().Center());
+        FetchLoginInfo(loginText, loginImage);
 
         auto pickers = new wxFlexGridSizer(2, wxSize(PX(5),PX(6)));
         pickers->AddGrowableCol(1);
         topsizer->Add(pickers, wxSizerFlags().Expand().PXDoubleBorderAll());
 
-        pickers->Add(new wxStaticText(this, wxID_ANY, _("Project:")),
-                     wxSizerFlags().CenterVertical().Right().BORDER_MACOS(wxTOP, 1));
+        pickers->Add(new wxStaticText(this, wxID_ANY, _("Project:")), wxSizerFlags().CenterVertical().Right());
         m_project = new wxChoice(this, wxID_ANY);
         pickers->Add(m_project, wxSizerFlags().Expand().CenterVertical());
 
-        pickers->Add(new wxStaticText(this, wxID_ANY, _("Language:")),
-                     wxSizerFlags().CenterVertical().Right().BORDER_MACOS(wxTOP, 1));
+        pickers->Add(new wxStaticText(this, wxID_ANY, _("Language:")), wxSizerFlags().CenterVertical().Right());
         m_language = new wxChoice(this, wxID_ANY);
         pickers->Add(m_language, wxSizerFlags().Expand().CenterVertical());
 
-        pickers->AddSpacer(PX(5));
-        pickers->AddSpacer(PX(5));
-
-        pickers->Add(new wxStaticText(this, wxID_ANY, _("File:")),
-                     wxSizerFlags().CenterVertical().Right().BORDER_MACOS(wxTOP, 1));
-        m_file = new wxChoice(this, wxID_ANY);
-        pickers->Add(m_file, wxSizerFlags().Expand().CenterVertical());
+        m_files = new CrowdinFileList(this);
+        topsizer->Add(m_files, wxSizerFlags(1).Expand().PXDoubleBorderAll());
 
         m_activity = new ActivityIndicator(this);
-        topsizer->AddSpacer(PX(5));
         topsizer->Add(m_activity, wxSizerFlags().Expand().PXDoubleBorder(wxLEFT|wxRIGHT));
-        topsizer->AddSpacer(PX(5));
+        topsizer->AddSpacer(MSW_OR_OTHER(PX(4), PX(2)));
 
         auto buttons = CreateButtonSizer(wxOK | wxCANCEL);
         auto ok = static_cast<wxButton*>(FindWindow(wxID_OK));
@@ -320,7 +454,6 @@ public:
         CenterOnParent();
 
         m_project->Bind(wxEVT_CHOICE, [=](wxCommandEvent&){ OnProjectSelected(); });
-        m_file->Bind(wxEVT_CHOICE, [=](wxCommandEvent&){ OnFileSelected(); });
         ok->Bind(wxEVT_UPDATE_UI, &CrowdinOpenDialog::OnUpdateOK, this);
         ok->Bind(wxEVT_BUTTON, &CrowdinOpenDialog::OnOK, this);
 
@@ -333,11 +466,40 @@ public:
     wxString OutLocalFilename;
 
 private:
+    void FetchLoginInfo(SecondaryLabel *label, AvatarIcon *icon)
+    {
+        label->Hide();
+        icon->Hide();
+
+        CrowdinClient::Get().GetUserInfo()
+        .then_on_window(this, [=](CrowdinClient::UserInfo u)
+        {
+            label->SetLabel(_("Signed in as:") + " " + u.name);
+            icon->SetUserName(u.name);
+            if (u.avatar.empty())
+            {
+                icon->Show();
+            }
+            else
+            {
+                http_client::download_from_anywhere(u.avatar)
+                .then_on_window(this, [=](downloaded_file f)
+                {
+                    icon->LoadIcon(f.filename());
+                    icon->Show();
+                });
+            }
+            Layout();
+            label->Show();
+        })
+        .catch_all([](dispatch::exception_ptr){});
+    }
+
     void EnableAllChoices(bool enable = true)
     {
         m_project->Enable(enable);
-        m_file->Enable(enable);
         m_language->Enable(enable);
+        m_files->Enable(enable);
     }
 
     void FetchProjects()
@@ -375,52 +537,25 @@ private:
         {
             m_activity->Start();
             EnableAllChoices(false);
-            CrowdinClient::Get().GetProjectInfo(m_projects[sel-1].identifier)
+            m_files->ClearFiles();
+            CrowdinClient::Get().GetProjectInfo(m_projects[sel-1].id)
                 .then_on_window(this, &CrowdinOpenDialog::OnFetchedProjectInfo)
-                .catch_all(m_activity->HandleError);
+                .catch_all([=](dispatch::exception_ptr e){
+                    m_activity->HandleError(e);
+                    EnableAllChoices(true);
+                });
         }
     }
 
     void OnFetchedProjectInfo(CrowdinClient::ProjectInfo prj)
     {
         m_info = prj;
-        // Put supported files first in the list:
-        std::vector<std::wstring> f_unsup;
-        m_info.files.clear();
-        m_supportedFilesCount = 0;
-        for (auto& i: prj.files)
-        {
-            if (IsFileSupported(i))
-            {
-                m_info.files.push_back(i);
-                m_supportedFilesCount++;
-            }
-            else
-            {
-                f_unsup.push_back(i);
-            }
-        }
-        std::move(f_unsup.begin(), f_unsup.end(), std::inserter(m_info.files, m_info.files.end()));
-
         m_language->Clear();
         m_language->Append("");
         for (auto& i: m_info.languages)
             m_language->Append(i.DisplayName());
 
-        m_file->Clear();
-        m_file->Append("");
-        for (auto& i: m_info.files)
-        {
-            if (IsFileSupported(i))
-            {
-                m_file->Append(i);
-            }
-            else
-            {
-                /// TRANSLATORS: This is a file selector list, %s is filename, and it is shown for Crowdin files not editable in Poedit
-                m_file->Append(wxString::Format(L"%s — not supported", i));
-            }
-        }
+        m_files->SetFiles(m_info.files);
 
         EnableAllChoices();
         m_activity->Stop();
@@ -445,48 +580,35 @@ private:
             }
         }
 
-        if (m_supportedFilesCount == 1)
-            m_file->SetSelection(1);
+        if (m_info.files.size() == 1)
+            m_files->SelectRow(0);
 
-        if (m_supportedFilesCount == 0)
-        {
-            m_activity->StopWithError(_("This project has no files that can be translated in Poedit."));
-            m_file->Disable();
-            m_language->Disable();
-        }
-    }
-
-    void OnFileSelected()
-    {
-        auto filesel = m_file->GetSelection();
-        if (filesel - 1 < m_supportedFilesCount)
-            m_activity->Stop();
-        else
-            m_activity->StopWithError(_(L"This file can only be edited in Crowdin’s web interface."));
     }
 
     void OnUpdateOK(wxUpdateUIEvent& e)
     {
-        auto filesel = m_file->GetSelection();
         e.Enable(!m_activity->IsRunning() &&
                  m_project->GetSelection() > 0 &&
                  m_language->GetSelection() > 0 &&
-                 filesel > 0 && filesel - 1 < m_supportedFilesCount);
+                 m_files->GetSelectedRow() != wxNOT_FOUND);
     }
 
     void OnOK(wxCommandEvent&)
     {
-        auto crowdin_prj = m_info.identifier;
-        auto crowdin_file = m_info.files[m_file->GetSelection() - 1];
+        auto crowdin_file = m_info.files[m_files->GetSelectedRow()];
         auto crowdin_lang = m_info.languages[m_language->GetSelection() - 1];
         LanguageDialog::SetLastChosen(crowdin_lang);
-        OutLocalFilename = CreateLocalFilename(crowdin_file, crowdin_lang);
+        OutLocalFilename = CreateLocalFilename(crowdin_file.id, crowdin_file.fileName, crowdin_lang, m_info.id, m_info.name);
 
         m_activity->Start(_(L"Downloading latest translations…"));
 
         auto outfile = std::make_shared<TempOutputFileFor>(OutLocalFilename);
         CrowdinClient::Get().DownloadFile(
-                crowdin_prj, crowdin_file, crowdin_lang,
+                m_info.id,
+                crowdin_lang,
+                crowdin_file.id,
+                std::string(wxFileName(crowdin_file.fileName, wxPATH_UNIX).GetExt().utf8_str()),
+                /*forceExportAsXliff=*/false,
                 outfile->FileName().ToStdWstring()
             )
             .then_on_window(this, [=]{
@@ -496,48 +618,132 @@ private:
             .catch_all(m_activity->HandleError);
     }
 
-    bool IsFileSupported(const wxString& name) const
+    wxString CreateLocalFilename(int fileId, std::string fileName, const Language& lang, int projectId, wxString projectName)
     {
-        return boost::ends_with(name, ".po") || boost::ends_with(name, ".pot");
-    }
+        // sanitize to be safe filenames:
+        std::replace_if(fileName.begin(), fileName.end(), boost::is_any_of("\\/:\"<>|?*"), '_');
+        std::replace_if(projectName.begin(), projectName.end(), boost::is_any_of("\\/:\"<>|?*"), '_');
 
-    wxString CreateLocalFilename(const wxString& name, const Language& lang)
-    {
-        wxString cache;
-    #if defined(__WXOSX__)
-        cache = wxGetHomeDir() + "/Library/Caches/net.poedit.Poedit";
-    #elif defined(__UNIX__)
-        if (!wxGetEnv("XDG_CACHE_HOME", &cache))
-            cache = wxGetHomeDir() + "/.cache";
-        cache += "/poedit";
-    #else
-        cache = wxStandardPaths::Get().GetUserDataDir() + wxFILE_SEP_PATH + "Cache";
-    #endif
+        const wxString dir = GetCrowdinCacheDir() + projectName + " - " + lang.Code();
+        wxFileName localFileName(dir + "/Crowdin." << projectId << '.' << fileId << ' ' << fileName);
 
-        cache += wxFILE_SEP_PATH;
-        cache += "Crowdin";
+        auto ext = localFileName.GetExt().Lower();
+        if (ext == "po")
+        {
+            // natively supported file format, will be opened as-is
+        }
+        else if (ext == "pot")
+        {
+            // POT files are natively supported, but need to be opened as PO to see translations
+            localFileName.SetExt("po");
+        }
+        else if (ext == "xliff" || ext == "xlf")
+        {
+            // Do nothing, we don't want to use double-extension like .xliff.xliff
+        }
+        else
+        {
+            // Everything else is exported as XLIFF and we do want, at least for now, to keep
+            // the old extension for clarity, e.g. *.strings.xliff or *.rc.xliff
+            localFileName.SetFullName(localFileName.GetFullName() + ".xliff");
+        }
 
-        if (!wxFileName::DirExists(cache))
-            wxFileName::Mkdir(cache, wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
+        if (!wxFileName::DirExists(localFileName.GetPath()))
+            wxFileName::Mkdir(localFileName.GetPath(), wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
 
-        auto basename = name.AfterLast('/').BeforeLast('.');
-
-        return wxString::Format("%s%c%s_%s_%s.po", cache, wxFILE_SEP_PATH, m_info.name, basename, lang.Code());
+        return localFileName.GetFullPath();
     }
 
 private:
     wxButton *m_ok;
-    wxChoice *m_project, *m_file, *m_language;
+    wxChoice *m_project, *m_language;
+    CrowdinFileList *m_files;
     ActivityIndicator *m_activity;
 
     std::vector<CrowdinClient::ProjectListing> m_projects;
     CrowdinClient::ProjectInfo m_info;
-    int m_supportedFilesCount;
 };
 
+
+bool ExtractCrowdinMetadata(CatalogPtr cat,
+                            Language *lang = nullptr,
+                            int *projectId = nullptr, int *fileId = nullptr,
+                            std::string *xliffRemoteFilename = nullptr)
+{
+    auto& hdr = cat->Header();
+
+    if (lang)
+    {
+        *lang = hdr.HasHeader("X-Crowdin-Language")
+                ? Language::TryParse(hdr.GetHeader("X-Crowdin-Language").ToStdWstring())
+                : cat->GetLanguage();
+    }
+
+    const auto xliff = std::dynamic_pointer_cast<XLIFFCatalog>(cat);
+    if (xliff)
+    {
+        if (xliff->GetXPathValue("file/header/tool//@tool-id") == "crowdin")
+        {
+            try
+            {
+                if (projectId)
+                    *projectId = std::stoi(xliff->GetXPathValue("file/@*[local-name()='project-id']"));
+                if (fileId)
+                    *fileId = std::stoi(xliff->GetXPathValue("file/@*[local-name()='id']"));
+                if (xliffRemoteFilename)
+                    *xliffRemoteFilename = xliff->GetXPathValue("file/@*[local-name()='original']");
+                return true;
+            }
+            catch(...)
+            {
+                wxLogTrace("poedit.crowdin", "Missing or malformatted Crowdin project and/or file ID");
+            }
+        }
+    }
+    
+    if (hdr.HasHeader("X-Crowdin-Project-ID") && hdr.HasHeader("X-Crowdin-File-ID"))
+    {
+        if (projectId)
+            *projectId = std::stoi(hdr.GetHeader("X-Crowdin-Project-ID").ToStdString());
+        if (fileId)
+            *fileId = std::stoi(hdr.GetHeader("X-Crowdin-File-ID").ToStdString());
+        return true;
+    }
+
+    static const std::wregex RE_CROWDIN_FILE(L"^Crowdin\\.([0-9]+)\\.([0-9]+) .*");
+    auto name = wxFileName(cat->GetFileName()).GetName().ToStdWstring();
+
+    std::wsmatch m;
+    if (std::regex_match(name, m, RE_CROWDIN_FILE))
+    {
+        if (projectId)
+            *projectId = std::stoi(m.str(1));
+        if (fileId)
+            *fileId = std::stoi(m.str(2));
+        return true;
+    }
+
+    return false;
+}
+
 } // anonymous namespace
 
 
+bool CanSyncWithCrowdin(CatalogPtr cat)
+{
+    return ExtractCrowdinMetadata(cat, nullptr);
+}
+
+
+bool ShouldSyncToCrowdinAutomatically(CatalogPtr cat)
+{
+    // TODO: This check is fragile and breaks of the path is non-normalized,
+    //       e.g. uses different case or is relative or differently normalized.
+    //       Good for use with files from Recent Files, but not much else
+    return cat->GetFileName().StartsWith(GetCrowdinCacheDir());
+}
+
+
 void CrowdinOpenFile(wxWindow *parent, std::function<void(wxString)> onLoaded)
 {
     if (!CrowdinClient::Get().IsSignedIn())
@@ -573,15 +779,15 @@ void CrowdinSyncFile(wxWindow *parent, std::shared_ptr<Catalog> catalog,
         return;
     }
 
-    const auto& header = catalog->Header();
-    auto crowdin_prj = header.GetHeader("X-Crowdin-Project");
-    auto crowdin_file = header.GetHeader("X-Crowdin-File");
-    auto crowdin_lang = header.HasHeader("X-Crowdin-Language")
-                        ? Language::TryParse(header.GetHeader("X-Crowdin-Language").ToStdWstring())
-                        : catalog->GetLanguage();
+    wxLogTrace("poedit.crowdin", "Crowdin syncing file ...");
 
     wxWindowPtr<CloudSyncProgressWindow> dlg(new CloudSyncProgressWindow(parent));
 
+    Language crowdinLang;
+    int projectId, fileId;
+    std::string xliffRemoteFilename;
+    ExtractCrowdinMetadata(catalog, &crowdinLang, &projectId, &fileId, &xliffRemoteFilename);
+
     auto handle_error = [=](dispatch::exception_ptr e){
         dispatch::on_main([=]{
             dlg->EndModal(wxID_CANCEL);
@@ -603,19 +809,31 @@ void CrowdinSyncFile(wxWindow *parent, std::shared_ptr<Catalog> catalog,
     // This must be done right after entering the modal loop (on non-OSX)
     dlg->CallAfter([=]{
         CrowdinClient::Get().UploadFile(
-                str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
+                projectId,
+                crowdinLang,
+                fileId,
+                std::string(wxFileName(catalog->GetFileName()).GetExt().utf8_str()),
                 catalog->SaveToBuffer()
             )
-            .then([=]{
+            .then([=]
+            {
+                wxFileName filename = catalog->GetFileName();
                 auto tmpdir = std::make_shared<TempDirectory>();
-                auto outfile = tmpdir->CreateFileName("crowdin.po");
+                auto outfile = tmpdir->CreateFileName("crowdin." + filename.GetExt());
 
                 dispatch::on_main([=]{
                     dlg->Activity->Start(_(L"Downloading latest translations…"));
                 });
 
+                if (!xliffRemoteFilename.empty())
+                    filename.Assign(xliffRemoteFilename, wxPATH_UNIX);
+
                 return CrowdinClient::Get().DownloadFile(
-                        str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
+                        projectId,
+                        crowdinLang,
+                        fileId,
+                        std::string(filename.GetExt().utf8_str()),
+                        /*forceExportAsXliff=*/!xliffRemoteFilename.empty(),
                         outfile.ToStdWstring()
                     )
                     .then_on_main([=]
@@ -636,18 +854,25 @@ void CrowdinSyncFile(wxWindow *parent, std::shared_ptr<Catalog> catalog,
     dlg->ShowWindowModal();
 }
 
+bool CrowdinSyncDestination::AuthIfNeeded(wxWindow* parent) {
+    return CrowdinClient::Get().IsSignedIn()
+            || CrowdinLoginDialog(parent).ShowModal() == wxID_OK;
+}
 
 dispatch::future<void> CrowdinSyncDestination::Upload(CatalogPtr file)
 {
-    const auto& header = file->Header();
-    auto crowdin_prj = header.GetHeader("X-Crowdin-Project");
-    auto crowdin_file = header.GetHeader("X-Crowdin-File");
-    auto crowdin_lang = header.HasHeader("X-Crowdin-Language")
-                        ? Language::TryParse(header.GetHeader("X-Crowdin-Language").ToStdWstring())
-                        : file->GetLanguage();
+    wxLogTrace("poedit.crowdin", "Uploading file: %s", file->GetFileName().c_str());
 
-    return CrowdinClient::Get().UploadFile(
-                str::to_utf8(crowdin_prj), str::to_wstring(crowdin_file), crowdin_lang,
+    Language crowdinLang;
+    int projectId, fileId;
+    ExtractCrowdinMetadata(file, &crowdinLang, &projectId, &fileId);
+
+    return CrowdinClient::Get().UploadFile
+            (
+                projectId,
+                crowdinLang,
+                fileId,
+                std::string(wxFileName(file->GetFileName()).GetExt().utf8_str()),
                 file->SaveToBuffer()
             );
 }
