@@ -28,6 +28,8 @@
 #include "version.h"
 #include "str_helpers.h"
 
+#include <cstdlib>
+
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <boost/iostreams/copy.hpp>
@@ -43,6 +45,9 @@
 #include <cpprest/http_msg.h>
 #include <cpprest/filestream.h>
 
+#include <regex>
+
+
 #ifdef _WIN32
     #include <windows.h>
     #include <winhttp.h>
@@ -68,7 +73,10 @@ using namespace web;
 using utility::string_t;
 using utility::conversions::to_string_t;
 
-#ifndef _UTF16_STRINGS
+#ifdef _UTF16_STRINGS
+inline string_t to_string_t(const wxString& s) { return s.ToStdWstring(); }
+#else
+inline string_t to_string_t(const wxString& s) { return s.ToStdString(); }
 inline string_t to_string_t(const std::wstring& s) { return str::to_utf8(s); }
 #endif
 
@@ -77,7 +85,7 @@ class gzip_compression_support : public http::http_pipeline_stage
 public:
     pplx::task<http::http_response> propagate(http::http_request request) override
     {
-        request.headers().add(http::header_names::accept_encoding, L"gzip");
+        request.headers().add(http::header_names::accept_encoding, _XPLATSTR("gzip"));
 
         return next_stage()->propagate(request).then([](http::http_response response) -> pplx::task<http::http_response>
         {
@@ -117,13 +125,7 @@ class http_client::impl
 public:
     impl(http_client& owner, const std::string& url_prefix, int flags)
         : m_owner(owner),
-          m_native
-          (
-              sanitize_url(url_prefix, flags)
-            #ifdef _WIN32
-              , get_client_config()
-            #endif
-          )
+          m_native(sanitize_url(url_prefix, flags), get_client_config())
     {
         #define make_wide_str(x) make_wide_str_(x)
         #define make_wide_str_(x) L ## x
@@ -139,55 +141,18 @@ public:
 
         std::shared_ptr<http::http_pipeline_stage> gzip_stage = std::make_shared<gzip_compression_support>();
         m_native.add_handler(gzip_stage);
-
-    #ifdef _WIN32
-        m_networkListManager = nullptr;
-        CoCreateInstance(CLSID_NetworkListManager, NULL, CLSCTX_ALL, IID_INetworkListManager, (LPVOID *)&m_networkListManager);
-    #endif
-    }
-
-    ~impl()
-    {
-    #ifdef _WIN32
-        if (m_networkListManager)
-            m_networkListManager->Release();
-    #endif
     }
 
     static string_t ui_language;
 
-    bool is_reachable() const
-    {
-    #ifdef _WIN32
-        if (m_networkListManager)
-        {
-            NLM_CONNECTIVITY result;
-            HRESULT hr = m_networkListManager->GetConnectivity(&result);
-            if (SUCCEEDED(hr))
-                return result & (NLM_CONNECTIVITY_IPV4_INTERNET|NLM_CONNECTIVITY_IPV6_INTERNET);
-        }
-        // manager fallback (IPv6 ignorant):
-        DWORD flags;
-        return ::InternetGetConnectedState(&flags, 0);
-    #else
-        return true; // TODO
-    #endif
-    }
-
     void set_authorization(const std::string& auth)
     {
         m_auth = std::wstring(auth.begin(), auth.end());
     }
 
-    dispatch::future<::json> get(const std::string& url)
+    dispatch::future<::json> get(const std::string& url, const headers& hdrs)
     {
-        http::http_request req(http::methods::GET);
-        req.headers().add(http::header_names::accept,     L"application/json");
-        req.headers().add(http::header_names::user_agent, m_userAgent);
-        req.headers().add(http::header_names::accept_language, ui_language);
-        if (!m_auth.empty())
-            req.headers().add(http::header_names::authorization, m_auth);
-        req.set_request_uri(to_string_t(url));
+        auto req = build_request(http::methods::GET, url, hdrs);
 
         return
         m_native.request(req)
@@ -198,43 +163,41 @@ public:
         });
     }
 
-    dispatch::future<void> download(const std::string& url, const std::wstring& output_file)
+    dispatch::future<downloaded_file> download(const std::string& url, const headers& hdrs)
     {
         using namespace concurrency::streams;
-        auto fileStream = std::make_shared<ostream>();
+
+        auto req = build_request(http::methods::GET, url, hdrs);
 
         return
-        fstream::open_ostream(to_string_t(output_file)).then([=](ostream outFile)
-        {
-            *fileStream = outFile;
-            http::http_request req(http::methods::GET);
-            req.headers().add(http::header_names::user_agent, m_userAgent);
-            req.headers().add(http::header_names::accept_language, ui_language);
-            if (!m_auth.empty())
-                req.headers().add(http::header_names::authorization, m_auth);
-            req.set_request_uri(to_string_t(url));
-            return m_native.request(req);
-        })
+        m_native.request(req)
         .then([=](http::http_response response)
         {
             handle_error(response);
-            return response.body().read_to_end(fileStream->streambuf());
-        })
-        .then([=](size_t)
-        {
-            return fileStream->close();
+
+            downloaded_file file(extract_attachment_filename(req, response));
+
+            return
+            fstream::open_ostream(to_string_t(file.filename().GetFullPath()))
+            .then([=](ostream outFile)
+            {
+                return
+                response.body().read_to_end(outFile.streambuf())
+                .then([=](size_t)
+                {
+                    return outFile.close();
+                });
+            })
+            .then([file{std::move(file)}]()
+            {
+                return file;
+            });
         });
     }
 
-    dispatch::future<::json> post(const std::string& url, const http_body_data& data)
+    dispatch::future<::json> post(const std::string& url, const http_body_data& data, const headers& hdrs)
     {
-        http::http_request req(http::methods::POST);
-        req.headers().add(http::header_names::accept,     L"application/json");
-        req.headers().add(http::header_names::user_agent, m_userAgent);
-        req.headers().add(http::header_names::accept_language, ui_language);
-        if (!m_auth.empty())
-            req.headers().add(http::header_names::authorization, m_auth);
-        req.set_request_uri(to_string_t(url));
+        auto req = build_request(http::methods::POST, url, hdrs);
 
         auto body = data.body();
         req.set_body(body, data.content_type());
@@ -250,6 +213,24 @@ public:
     }
 
 private:
+    http::http_request build_request(http::method method, const std::string& relative_url, const headers& hdrs)
+    {
+        http::http_request req(method);
+        req.set_request_uri(to_string_t(relative_url));
+
+        req.headers().add(http::header_names::accept, _XPLATSTR("application/json"));
+        req.headers().add(http::header_names::user_agent, m_userAgent);
+        req.headers().add(http::header_names::accept_language, ui_language);
+        if (!m_auth.empty())
+            req.headers().add(http::header_names::authorization, m_auth);
+        for (const auto& h: hdrs)
+        {
+            req.headers().add(to_string_t(h.first), to_string_t(h.second));
+        }
+
+        return req;
+    }
+
     // handle non-OK responses:
     void handle_error(http::http_response r)
     {
@@ -258,7 +239,7 @@ private:
 
         int status_code = r.status_code();
         std::string msg;
-        if (r.headers().content_type() == L"application/json")
+        if (r.headers().content_type() == _XPLATSTR("application/json"))
         {
             try
             {
@@ -279,20 +260,50 @@ private:
         return to_string_t(url);
     }
 
-#ifdef _WIN32
-    // prepare WinHttp configuration
+    static std::string extract_attachment_filename(const http::http_request& request, const http::http_response& response)
+    {
+        // extract from Content-Disposition attachment filename:
+        auto hdr = response.headers().find(http::header_names::content_disposition);
+        if (hdr != response.headers().end())
+        {
+            static const std::basic_regex<utility::char_t> RE_FILENAME(_XPLATSTR("attachment; *filename=\"(.*)\""), std::regex_constants::icase);
+            std::match_results<string_t::const_iterator> match;
+            if (std::regex_search(hdr->second, match, RE_FILENAME))
+                return str::to_utf8(match.str(1));
+        }
+
+        // failing that, use the URL:
+        auto path = request.absolute_uri().path();
+        auto slash = path.find_last_of('/');
+        if (slash != string_t::npos)
+            path = path.substr(slash + 1);
+
+        return str::to_utf8(path);
+    }
+
     static http::client::http_client_config get_client_config()
     {
         http::client::http_client_config c;
-   
+
+    #ifdef _WIN32
+        // prepare WinHttp configuration:
         c.set_nativesessionhandle_options([](http::client::native_handle handle){
             DWORD dwOption = WINHTTP_PROTOCOL_FLAG_HTTP2;
             WinHttpSetOption(handle, WINHTTP_OPTION_ENABLE_HTTP_PROTOCOL, &dwOption, sizeof(dwOption));
         });
+    #else
+        // setup proxy on Unix platforms:
+        const char *proxy = std::getenv("https_proxy");
+        if (!proxy)
+            proxy = std::getenv("http_proxy");
+        if (proxy)
+            c.set_proxy(web::uri(proxy));
+    #endif
 
         return c;
     }
 
+#ifdef _WIN32
     static std::wstring windows_version()
     {
         OSVERSIONINFOEX info = { 0 };
@@ -305,8 +316,6 @@ private:
 
         return std::to_wstring(info.dwMajorVersion) + L"." + std::to_wstring(info.dwMinorVersion);
     }
-
-    INetworkListManager *m_networkListManager;
 #endif
 
     http_client& m_owner;
@@ -333,27 +342,84 @@ void http_client::set_ui_language(const std::string& lang)
     impl::ui_language = to_string_t(lang);
 }
 
-bool http_client::is_reachable() const
-{
-    return m_impl->is_reachable();
-}
-
 void http_client::set_authorization(const std::string& auth)
 {
     m_impl->set_authorization(auth);
 }
 
-dispatch::future<::json> http_client::get(const std::string& url)
+dispatch::future<::json> http_client::get(const std::string& url, const headers& hdrs)
 {
-    return m_impl->get(url);
+    return m_impl->get(url, hdrs);
 }
 
-dispatch::future<void> http_client::download(const std::string& url, const std::wstring& output_file)
+dispatch::future<downloaded_file> http_client::download(const std::string& url, const headers& hdrs)
 {
-    return m_impl->download(url, output_file);
+    return m_impl->download(url, hdrs);
 }
 
-dispatch::future<::json> http_client::post(const std::string& url, const http_body_data& data)
+dispatch::future<::json> http_client::post(const std::string& url, const http_body_data& data, const headers& hdrs)
 {
-    return m_impl->post(url, data);
+    return m_impl->post(url, data, hdrs);
+}
+
+
+#ifdef _WIN32
+
+class http_reachability::impl
+{
+public:
+    impl()
+    {
+        m_networkListManager = nullptr;
+        CoCreateInstance(CLSID_NetworkListManager, NULL, CLSCTX_ALL, IID_INetworkListManager, (LPVOID *)&m_networkListManager);
+    }
+
+    ~impl()
+    {
+        if (m_networkListManager)
+            m_networkListManager->Release();
+    }
+
+    bool is_reachable() const
+    {
+        if (m_networkListManager)
+        {
+            NLM_CONNECTIVITY result;
+            HRESULT hr = m_networkListManager->GetConnectivity(&result);
+            if (SUCCEEDED(hr))
+                return result & (NLM_CONNECTIVITY_IPV4_INTERNET|NLM_CONNECTIVITY_IPV6_INTERNET);
+        }
+        // manager fallback (IPv6 ignorant):
+        DWORD flags;
+        return ::InternetGetConnectedState(&flags, 0);
+    }
+
+private:
+    INetworkListManager *m_networkListManager;
+};
+
+#else // !WIN32
+
+class http_reachability::impl
+{
+public:
+    impl() {}
+    bool is_reachable() const { return true; } // TODO
+};
+
+#endif
+
+
+http_reachability::http_reachability(const std::string& /*url*/)
+    : m_impl(new impl)
+{
+}
+
+http_reachability::~http_reachability()
+{
+}
+
+bool http_reachability::is_reachable() const
+{
+    return m_impl->is_reachable();
 }
