@@ -553,7 +553,12 @@ public:
 AnyTranslatableTextCtrl::AnyTranslatableTextCtrl(wxWindow *parent, wxWindowID winid, int style)
    : CustomizedTextCtrl(parent, winid, style)
 {
-    m_attrs.reset(new Attributes(this));
+    ColorScheme::SetupWindowColors(this, [=]
+    {
+        m_attrs.reset(new Attributes(this));
+        HighlightText();
+    });
+
     Bind(wxEVT_TEXT, [=](wxCommandEvent& e){
         e.Skip();
         HighlightText();
@@ -599,6 +604,15 @@ void AnyTranslatableTextCtrl::SetLanguage(const Language& lang)
 
     ::SendMessage((HWND)GetHWND(), EM_SETEDITSTYLE, lang.IsRTL() ? SES_BIDI : 0, SES_BIDI);
 
+    CHARFORMAT2 cf;
+    ::ZeroMemory(&cf, sizeof(cf));
+    cf.cbSize = sizeof(cf);
+    cf.dwMask = CFM_LCID;
+    cf.lcid = LocaleNameToLCID(str::to_wstring(m_language.LanguageTag()).c_str(), 0);
+    if (cf.lcid == 0)
+        cf.lcid = LOCALE_USER_DEFAULT;
+    ::SendMessage((HWND)GetHWND(), EM_SETCHARFORMAT, SCF_ALL, (LPARAM)&cf);
+
     UpdateRTLStyle();
 #endif
 }
